# SPARQLprov

SPARQLprov is a query rewriting based tool to compute the provenance
of SPARQL answers

## Compiling the code

First, compile the code in folder +rewiring+ (see the +CMakeLists.txt+
for compilation requirements).  To build the code into the `build`
folder follow the next commands inside the folder of this repository:

```
mkdir build
cd build
cmake ../
```

After compiling the code you can test it with the following command
(inside the `build` folder).

```
cmake test
```

Alternatively, you can execute any of the created executable tests.
For instance, the following command run only tests for class
`Polynomial`.

```
./SPMPolynomialTest
```

## Using the query rewriting

This repository contains several example queries (in the folder
`queries`) that can be used to illustrate the query rewritten process.
The `rewritter` receives the name of a reification scheme along with
the name of the file that contains the query.  For instance:

```bash
./rewritter n ../queries/watdiv/C1.sparql
```

In this case `n` is the name of the reification scheme and
`../queries/watdiv/C1.sparql` is the file that contains the query.
The followed are the supported reification scheme names:

- `n`: named graphs scheme.
- `s`: standard reification scheme.
- `w`: Wikidata scheme.
- `v`: Wikidata variant scheme.

Let `u` be the URL used to reify a triple pattern.  The named graph
scheme transforms a triple pattern `s p o` into the following pattern:

```
GRAPH u { s p o }
```

Similarly, the standard reification scheme returns the pattern:

```
u rdf:subject s ; rdf:predicate p ; rdf:object o .
```

where `rdf` is the following prefix
```
http://www.w3.org/1999/02/22-rdf-syntax-ns#
```

The Wikidata and Wikidata variant reification schemes return a pattern
of the form:

```
s p1 u . u p2 o .
```

where `p1` and `p2` are URLs created from the URL of predicate `p`.
In the Wikidata reification scheme the URL of predicate `p` has the
form:

```
<http://www.wikidata.org/prop/direct/P20>
```

The predicates `p1` and `p2` are created respectively by replacing in
the predicate the string `/direct/` by `/` and `/statement/`.

On the other hand, the Wikidata variant does not assume a particular
form for the predicate `p` and can be used for any RDF dataset.  The
predicates `p1` and `p2` result from replacing the string `http://` by
`http://example.org/p/` and `http://example.org/ps/`.

## Decoding annotated answers

The afforementioned rewritten queries return answers extended with
values that encode their provenance.  For instance, the file
`query_answers/watdiv/L3.csv` contains a subset of the mappings
returned by the rewritten version of query L3.  This result encodes a
the provenance polynomial of each answer.  The following command shows
how this result is displayed:

```
ruby/bin/decode_polynomials.rb ../query_answers/watdiv/L3.csv
{:v0=>"http://db.uwaterloo.ca/~galuc/wsdbm/User43337"} =>
  ( + ( * ( + <http://example.org/statement/53412926> )
          ( + <http://example.org/statement/53412932> ) ) )
{:v0=>"http://db.uwaterloo.ca/~galuc/wsdbm/User50045"} =>
  ( + ( * ( + <http://example.org/statement/60019210> )
          ( + <http://example.org/statement/60019225> ) )
      ( * ( + <http://example.org/statement/60019211> )
          ( + <http://example.org/statement/60019225> ) ) )
```

Similarly, the lineage of each answer can be obtained with the
following command.

```
ruby/bin/decode_lineage.rb ../query_answers/watdiv/L3.csv
{:v0=>"http://db.uwaterloo.ca/~galuc/wsdbm/User43337"} =>
  ["http://example.org/statement/53412926",
   "http://example.org/statement/53412932"]
{:v0=>"http://db.uwaterloo.ca/~galuc/wsdbm/User50045"} =>
  ["http://example.org/statement/60019210",
   "http://example.org/statement/60019225",
   "http://example.org/statement/60019211"]
```
