#!/usr/bin/env ruby
# frozen_string_literal: true

require 'csv'
require 'linkeddata'
require 'sparql'
require 'sparql/client'
require 'pry'
require_relative '../lib/polynomial_table'

# This is the File where RDF data is stored
repository_file = ARGV[0]

repository = RDF::Repository.load(repository_file, format: :nquads)

# This is where the query is loaded
# Each dataset is associated to a query
query_file = repository_file
               .sub('dataset_examples', 'queries')
               .sub(/\.nt$/, '.sparql')
               .sub(/\.nq$/, '.sparql')

# We rewritten the query
puts "Query file: #{query_file}"
rewritten_query = `build/rewrite n #{query_file}`
puts ''

puts rewritten_query

sse = SPARQL.parse(rewritten_query)

puts 'Rewritten query SSE:'
puts sse.to_sse
puts ''

puts 'Rewritten query Answers:'

solutions = repository.query(sse)
puts solutions.to_csv
puts ''

# Decode solutions as polynomials
puts 'Rewritten K-relation:'
csv_table = CSV.parse(solutions.to_csv, headers: true)

k_relation = SPARQLprov.decode(csv_table)

k_relation.each do |answer, polynomial|
  puts "#{answer.inspect} => #{polynomial}"
end
