# frozen_string_literal: true

desc 'Create Fuseki 3 container'
named_task 'image_fuseki' do
  container = 'fuseki3-ubuntu2004'

  puts "Lauching container #{container}"
  launch_ubuntu_container(container, '20.04')

  puts 'Install required system packages'
  sh "lxc exec #{container} -- apt update"
  sh "lxc exec #{container} -- apt -y upgrade"
  sh "lxc exec #{container} -- apt install -y default-jdk daemonize net-tools"

  tars = %w[apache-jena-3.17.0.tar.gz apache-jena-fuseki-3.17.0.tar.gz].freeze
  archive = 'https://archive.apache.org/dist/jena/binaries'

  tars.each do |tar|
    sh "lxc exec #{container} -- su - ubuntu -c 'curl #{archive}/#{tar} --output #{tar}'"
    sh "lxc exec #{container} -- su - ubuntu -c 'tar -xzf #{tar}'"
    sh "lxc exec #{container} -- su - ubuntu -c 'rm #{tar}'"
  end

  sh "lxc exec #{container} -- su - ubuntu -c 'mkdir /home/ubuntu/dataset'"
  sh "lxc exec #{container} -- su - ubuntu -c 'mkdir /home/ubuntu/tdb'"

  fuseki_config = 'config/fuseki/fuseki-config.ttl'
  sh "lxc file push #{fuseki_config} #{container}/home/ubuntu/fuseki-config.ttl"

  puts 'Create the container image'
  sh "lxc stop #{container}"
  sh "lxc publish #{container} --alias #{container}"
end

desc 'Create Fuseki 3 container with union graph'
named_task 'image_fuseki_union_graph', task_dependency('image_fuseki') do
  container = 'fuseki3-ubuntu2004-union-graph'

  sh "lxc launch fuseki3-ubuntu2004 #{container}"

  fuseki_config = 'config/fuseki/fuseki-config-union-graph.ttl'
  sh "lxc file push #{fuseki_config} #{container}/home/ubuntu/fuseki-config.ttl"

  puts 'Create the container image'
  sh "lxc stop #{container}"
  sh "lxc publish #{container} --alias #{container}"
end

desc 'Create Fuseki 3 container'
task image_fuseki: task_dependency('image_fuseki')

desc 'Create Fuseki 3 container with union graph'
task image_fuseki_union_graph: task_dependency('image_fuseki_union_graph')
