# frozen_string_literal: true

desc 'Create Virtuoso 7 container image'
named_task 'image_virtuoso' do
  container = 'virtuoso7-debian9'

  puts "Lauching container #{container}"
  launch_debian_container(container, '9')

  puts 'Install required system packages'
  sh "lxc exec #{container} -- apt update"
  sh "lxc exec #{container} -- apt -y upgrade"
  sh "lxc exec #{container} -- apt install -y libssl1.0-dev build-essential bison flex " \
    'autotools-dev automake libtool gperf gawk wget unzip'

  To speed up sessions
  sh 'lxc exec virtuoso7-debian9 -- sed -i ' \
     "'s/session[[:space:]]optional[[:space:]]pam_systemd.so//' /etc/pam.d/common-session"

  puts 'Getting virtuoso code'
  code_url = 'https://github.com/openlink/virtuoso-opensource/archive/refs/tags/v7.2.5.1.zip'
  sh "lxc exec #{container} -- su - debian -c 'wget #{code_url}'"
  sh "lxc exec #{container} -- su - debian -c 'unzip v7.2.5.1.zip'"
  sh "lxc exec #{container} -- su - debian -c 'rm v7.2.5.1.zip'"
  sh "lxc exec #{container} -- su - debian -c 'mv virtuoso-opensource-7.2.5.1 virtuoso-opensource'"

  puts 'Setup Virtuoso'
  sh "lxc exec #{container} -- su - debian -c " \
     "'export CFLAGS=\"-O2 -m64\" && cd virtuoso-opensource && " \
     './autogen.sh && ' \
     './configure --prefix=/home/debian/virtuoso-7.2.5.1 && ' \
     "make && make install'"

  sh "lxc exec #{container} -- su - debian -c 'mkdir /home/debian/dataset'"
  sh 'lxc file push config/virtuoso/virtuoso.ini ' \
     "#{container}/home/debian/virtuoso-7.2.5.1/var/lib/virtuoso/db/"
  puts 'Virtuoso compiled and installed'

  # Publish the container
  puts 'Create a container image'
  system "lxc stop #{container}"
  system "lxc publish #{container} --alias #{container}"
end

desc 'Create Virtuoso 7 container image'
task image_virtuoso: task_dependency('image_virtuoso')
