# frozen_string_literal: true

require 'fileutils'

TPCH_HOME = File.dirname(File.join(__dir__))
TPCH_DBGEN_DIR = File.join(TPCH_HOME, 'tpch-tool/2.18.0_rc2/dbgen')
TPCH_DSS_QUERY = File.join(TPCH_HOME, 'query_parameter_templates')
TPCH_QGEN = "#{TPCH_DBGEN_DIR}/qgen".freeze

tpch_parameter_tasks = []
tpch_query_tasks = []

%w[01 03 05 06 07 08 09 10 12 14 19].each do |query|
  SCALES.each do |scale_factor|
    params_file = File.join('tpch/query_parameters',
                            "q#{query}_s#{scale_factor.sub('.', 'd')}.rb")
    params_task = "tpch_params_#{query}_#{scale_factor}"
    tpch_parameter_tasks.append(params_task)

    desc "Generate parameters for query=#{query}, scale factor=#{scale_factor}"
    task params_task => params_file

    file params_file do
      FileUtils.mkdir_p(File.dirname(params_file))
      sh "echo 'PARAMS = [' > #{params_file}"
      (1..20).each do |j|
        sh "DSS_CONFIG=#{TPCH_DBGEN_DIR} DSS_QUERY=#{TPCH_DSS_QUERY} " \
           "#{TPCH_QGEN} -r #{j} -s #{scale_factor} #{query} | " \
           "sed --expression='s/^--/  #/g' >> #{params_file}"
      end
      sh "echo ']' >> #{params_file}"
    end

    %w[base base_non_aggregate prov_naryrel prov_naryrel_non_aggregate].each do |mode|
      (1..10).each do |j|
        query_file = File.join('tpch/queries',
                               query, mode, scale_factor.sub('.', 'd'),
                               format('q%03d.sparql', j))

        query_task = :"query_#{query}_#{scale_factor}_#{mode}_#{'%03d' % j}"
        tpch_query_tasks.append(query_task)

        desc "Generate query=#{query}, mode=#{mode}, scale factor=#{scale_factor}"
        task query_task => query_file

        query_generator = File.join('tpch/query_generators', "#{query}.rb")
        query_example = File.join('tpch/sparql_examples',
                                  "#{query}_#{mode}.sparql")

        file query_file => [params_file, query_example, query_generator] do
          FileUtils.mkdir_p(File.dirname(query_file))
          sh "#{query_generator} #{mode} #{scale_factor.sub('.', 'd')} #{j} " \
             "> #{query_file}"
        end
      end
    end
  end
end

desc 'Generate TPCH query parameters'
task tpch_parameters: tpch_parameter_tasks

desc 'Generate TPCH queries'
task tpch_queries: tpch_query_tasks
