# frozen_string_literal: true

[
  {
    name: 'namedgraphs',
    files: [["watdiv.#{WATDIV_SIZE}M-namedgraphs.nt.gz", "watdiv.#{WATDIV_SIZE}M-namedgraphs.nq.gz"]],
    image: 'fuseki3-ubuntu2004-union-graph'
  },
  {
    name: 'rdf',
    files: [
      ["watdiv.#{WATDIV_SIZE}M-rdf.nt.gz", "watdiv.#{WATDIV_SIZE}M-rdf.nt.gz"],
      ["watdiv.#{WATDIV_SIZE}M.nt.gz", "watdiv.#{WATDIV_SIZE}M.nt.gz"]
    ],
    image: 'fuseki3-ubuntu2004'
  },
  {
    name: 'wikidata',
    files: [
      ["watdiv.#{WATDIV_SIZE}M-wikidata.nt.gz", "watdiv.#{WATDIV_SIZE}M-wikidata.nt.gz"],
      ["watdiv.#{WATDIV_SIZE}M.nt.gz", "watdiv.#{WATDIV_SIZE}M.nt.gz"]
    ],
    image: 'fuseki3-ubuntu2004'
  }
].each do |scheme|
  task_description =
    "Create WatDiv #{WATDIV_SIZE}M Fuseki 3 container for #{scheme[:name]} reification scheme"
  task_name = "create_watdiv_#{WATDIV_SIZE}M_#{scheme[:name]}_fuseki_3_container"

  desc task_description
  named_task task_name do
    image = scheme[:image]
    container = "watdiv-#{WATDIV_SIZE}M-#{scheme[:name]}-#{image}"
    user = 'ubuntu'
    system "lxc launch #{image} #{container}"
    scheme[:files].each do |file_pair|
      system "lxc file push watdiv/datasets/#{file_pair[0]} #{container}/home/#{user}/dataset/#{file_pair[1]}"
    end
    puts 'Loading data'
    sh "lxc exec #{container} -- su - ubuntu -c " \
       "'apache-jena-3.17.0/bin/tdbloader2 --loc tdb dataset/* > load-1.log 2> load-2.log'"
  end

  desc task_description
  task "create_watdiv_#{scheme[:name]}_fuseki": task_dependency(task_name)
end
