# frozen_string_literal: true

require_relative '../../lib/load_virtuoso'

%w[fuseki virtuoso].each do |engine|
  task_description = "Create Wikidata container for engine=#{engine}"
  task_name = "create_wikidata_#{engine}"

  desc task_description
  named_task task_name do
    setting = {
      'fuseki' => { image: 'fuseki3-ubuntu2004', user: 'ubuntu' },
      'virtuoso' => { image: 'virtuoso7-debian9', user: 'debian' }
    }
    image = setting[engine][:image]
    user = setting[engine][:user]
    container = "wikidata-20200127-#{image}"
    sh "lxc launch #{image} #{container}"
    File['wikidata/dataset/*.nt.gz'].each do |file|
      system "lxc file push #{file} #{container}/home/#{user}/dataset"
    end
    puts 'Loading data'
    if engine == 'fuseki'
      sh "lxc exec #{container} -- su - #{user} -c " \
         "'apache-jena-3.17.0/bin/tdbloader2 --loc tdb dataset/* > load-1.log 2> load-2.log'"
    else
      virtuoso_7_load_dataset(container)
    end
  end

  desc task_description
  task task_name
end
